close all; clc;

% Script paramenters
M = 512;
N = 512;
B = 31;
imageName = 'feathers';
sigma_noise = 0.05;
runNLM = false;

% Average intensity parameters
numSuperpixels = 3000;

% Bilateral filter parameters
sigmaSpace = 1.5;
sigmaIntensity = 0.2;  

% Non-local means parameters
sigmaSpace_nlm          = 1.0;
nlm_sigma               = 0.1;
searchWindowRadius      = 7;


%% ------------------------------------------------------------------------
%Load multispectral image - X is original clean image
X = zeros(M,N,B);
for b=1:B
    filename = [imageName,'_ms/', imageName,'_ms_',num2str(b,'%02d'),'.png'];
    X(:,:,b) = im2double(imread(filename));    
end

%% ------------------------------------------------------------------------
% Add noise - Y is the noisy image
addedNoise = sigma_noise.*randn(size(X));
Y = X + addedNoise;


%% ------------------------------------------------------------------------
% Proposed method
fprintf('Denoising with proposed method...\n');
p = zeros(B,1);
[ Xr, L ] = averageIntensity( Y, numSuperpixels );
for i=1:B
    p(i) = PSNR(X(:,:,i),Xr(:,:,i),0);
end
avgP = mean(p);

fprintf('Average PSNR for proposed method: %f\n', avgP);
showSuperpixels(X(:,:,19),L);

%% -----------------
% Bilateral filter

fprintf('Denoising with bilateral filter...\n');

Xr_blt = zeros(size(Y));
p_blt = zeros(B,1);
filterSize  = 2*sigmaSpace+1;
averageFilterRadius     = floor(filterSize/2);
for c=1:B
    Xr_blt(:,:,c) = bilateral(Y(:,:,c), averageFilterRadius, sigmaSpace, sigmaIntensity);
    p_blt(c) = PSNR(X(:,:,c),Xr_blt(:,:,c),0);
end 
avgP_blt = mean(p_blt);


fprintf('Average PSNR for bilateral filter: %f\n', avgP_blt);

%% -----------------
% Non-local means

if (runNLM == true)
    fprintf('Denoising with non-local means...\n');

    filterSize_nlm  = 2*sigmaSpace_nlm+1;
    averageFilterRadius_nlm     = floor(filterSize_nlm/2);

    Xr_nlm = zeros(size(Y));
    p_nlm = zeros(B,1);
    for c=1:B
        Xr_nlm(:,:,c) = nlm(Y(:,:,c), searchWindowRadius, averageFilterRadius_nlm, sigmaSpace_nlm, nlm_sigma);
        p_nlm(c) = PSNR(X(:,:,c),Xr_nlm(:,:,c),0);
    end
    avgP_nlm = mean(p_nlm);
    fprintf('Average PSNR for non-local means: %f\n', avgP_nlm);
end

%% Plot results

nCols = 2;
if (runNLM == true)
    nCols = 3;
end
figure;
subplot(2,nCols,1);
imshow(X(:,:,19))
title('Original image - band 19');
subplot(2,nCols,2);
imshow(Y(:,:,19))
title('Noisy image - band 19');
subplot(2,nCols,3);
imshow(Xr_blt(:,:,19))
title('Bilateral filter - band 19');
if (runNLM == true)
    subplot(2,nCols,4);
    imshow(Xr_nlm(:,:,19))
    title('Non-Local Means - band 19');
    subplot(2,nCols,5);
    imshow(Xr(:,:,19))
    title('Proposed method - band 19');
else
    subplot(2,nCols,4);
    imshow(Xr(:,:,19))
    title('Proposed method - band 19');
end

